#include <arma.h>

#ifndef ARFIMA_INCLUDED
#define ARFIMA_INCLUDED

#include <database.h>
#include <modelbase.h>
#include <maximize.h>

enum
{   Z_VAR = X_VAR + 1, W_VAR
};
enum
{   M_MAXLIK, M_NLS, M_MAXMPLIK, M_INITONLY, M_NLS_STATIONARY
};

#ifndef NO_DLL
  #ifdef PCGIVE
    extern "pcgive10,FnFracSigma"
  #else
    extern "arfima,FnFracSigma"
  #endif
#endif
fracsigma(const vtheta, const p, const q, const dvar, const ct, const avsig);

/*-------------------------- Arfima : Modelbase ----------------------------*/
class Arfima : Modelbase
{
	decl m_asMethods;									   /* method labels */
	decl m_iModel;								    /* current model number */
    decl m_cZ;                  /* number of innovative regressors (if any) */
    decl m_cW;                       /* 0, or 1 (using weighted estimation) */
	decl m_asY, m_asX;							   /* Y and parameter names */

    decl m_cAR;            /* p: number of parameters in AR part, default 0 */
    decl m_cMA;            /* q: number of parameters in MA part, default 0 */
	decl m_cFuncEval;          			  /* number of function evaluations */
    decl m_fDfixed;                                                /* fix d */
    decl m_mZ;               /* innovation regressor data vector [cT][m_cX] */
    decl m_mW;                          /* 1 or vector with weights [cT][1] */
    decl m_fKnownMean;                                  /* TRUE: known mean */
    decl m_dYmean;/* mean of data, known or actual, depends on m_fKnownMean */
    decl m_dYvar;                                       /* variance of data */
    decl m_dLogVar;                                /* log of error variance */
	decl m_dLogWeight;		/* sum of log weights (for weighted estimation) */
	decl m_bConcentrateX;	   /* TRUE: X's are concentrated out in EML/MPL */
	decl m_mForc, m_mForcLevel;			   /* for storage after forecasting */
	decl m_mForcExp;	  /* exp(forecasts), of level if these are computed */
    decl m_mResidual;		               /* matrix of residuals [m_cT][1] */
	decl m_cTdynforc;

	Arfima();                                                /* constructor */
	virtual GetPackageName();
	virtual GetPackageVersion();
    Arma(const cAR, const cMA);/* sets no of coeffs in AR and MA polynomial */
	ARMA(const cAR, const cMA);										/* same */
	virtual GetXNames();
	virtual GetXBeta(const vP);
	virtual GetXBetaStart(const mY);
	virtual GetXSizeInit();
	virtual GetXBetaForc(const vP, const cTforc);
	virtual GetXPartial();
	virtual GetZNames();
	virtual GetZGamma(const vP);
	virtual GetZGammaStart(const mY);
	virtual GetZSizeInit();
	virtual GetParNames();
	GetForcData(const iGroup, const cTforc);
    GetD();                                 /* returns estimated value of d */
	virtual GetcDfLoss();
    FixD(const dD);                                    /* fix d at value dD */
    FixAR(const iOrder, const valAR);           /* 1 to fix AR(1) term, etc. */
    FixMA(const iOrder, const valMA);          /* 1 to fix MA(1) term, etc. */
    FreeD();                                 /* estimate d freely (default) */
	FreeARMA();			   /* estimate all arma parameters freely (default) */
	FreeArma();
	FixX();
	FreeX();
	GetParStatus();  /* ret: {fDfixed,cAR,cMA,mIsFree,mFixedPar,fKnownMean} */
	GetFuncEval();             /* returns number of likelihood evaluations  */
	SetFuncEval(       );         /* sets number of likelihood evaluations  */
    FixMean(const dYmean);   /* use to set known mean (used in simulations) */
    SetKnownMean(const dYmean);								 /* use FixMean */
	GetMean();		/* returns actual mean (after estimation) or known mean */
    FreeMean();               			             /* use the sample mean */
    UseSampleMean();           			             /* use the sample mean */
	virtual SetStartPar(const vParFree);
    SetSigma2(const dSigma);      /* sets a value for the residual variance */
    GetSigma2();                              /* gets the residual variance */

	InitData();                                 /* get data and standardize */
	static EstimateGPH(const mY, const iTrunc, const fPrint);
	static EstimateGSP(const mY, const iTrunc, const fPrint);
	static SolveAR(const mY, const cP, const cQ, const mIsFree, const fPrint, ...);
	static SolveMA(const mY, const cQ, const mIsFree, const dItEps,
		const cItMax, const fPrint);
	static Acf(const cT, const vP, const cAR, const cMA, const bPrintErr);
	static FlipMA(const vMA);/* returns adjusted params and # roots outside */
    virtual InitPar();			   /* get starting values before estimation */
	IsStationary(const mTheta, const dD);
	Filter(const mY, const dYmean, const vPar, const cAR, const cMA,
		const mXBeta, const mZGamma, const adLogDet, ...); /* arfima filter */
	FilterNaive(const mY, const dYmean,	const vPar, const cAR, const cMA,
		const mXBeta, const mZGamma, const adLogDet, ...);	  /* NLS filter */
    fLogLik(const vP, const adFunc, const avScore, const amHessian);
    fNLS(const vP, const adFunc, const avScore, const amHessian);
	GetModelLabel();
	GetMethodLabel();
	virtual DoEstimation(vPar);
	static CumForc(const dYlevel, mYforc, mYact, mMaCoefs, const dSigma2,
		mYforcOpt, mForcVar);						 /* cumulates forecasts */
    Forecast(const cTforc, ...);				 		   /* get forecasts */
	virtual GetResiduals();		 /* get residuals from used filter function */
	GetNaiveResiduals();						     /* get naive residuals */
    virtual Covar();							/* computes variance matrix */
    virtual Output();/*print results, if m_fPrint==TRUE,automatically called*/
	virtual TestGraphicAnalysis();
	virtual TestForecastGraphics(const bExp);
	virtual TestSummary();
	// OxPack
	DoStore(const sOpt, const fQuery);
	virtual SendVarStatus();
	virtual ReceiveModel();
	virtual SendDialog(const sDialog);
	virtual SendMethods();
	virtual SendMenu(const sMenu);
	virtual ReceiveDialog(const sDialog, const asOptions, const aValues);
	GetSettingValues();
	virtual GetModelSettings();
	virtual SetModelSettings(const aValues);
};
/*------------------------ END Arfima : Modelbase --------------------------*/

#endif /* ARFIMA_INCLUDED */
